//
//  RapidBitalino.cpp
//  Bitalino
//
//  Created by James on 20/11/2017.
//
//

#include "RapidBitalino.h"

RapidBitalino::RapidBitalino ( void )
{
}

RapidBitalino::~RapidBitalino ( void )
{
}

void RapidBitalino::clearTrainingData ( void )
{
    // Clear the training set
    (*currentPhrase).elements.clear();
    printf("Cleared training data\n");
}

void RapidBitalino::recordTrainingData( uint32_t samplesToCollect, double output )
{
    this->samplesToCollect = samplesToCollect;
    gatheringTrainingData = true;
    bitalinoThread.data.reset(); // Reset read/write pointers
    currentTrainingOutput = { output };
    printf("Recoriding training input for output: %f\n", output);
}

void RapidBitalino::setup ( void )
{
    // Get a reference to a new phrase which stores our trainingData
    currentPhrase = trainingData.createNewPhrase("BITalino");
    // Set threaded train callback: (not sync)
    rapidMixThread.threadedTrainCallback = [&] (selectedTraining* t )
    {
        if (trainingData.trainingSet.size() > 0)
            t->train(trainingData);
    };
    // Set callback for when model is done training (sync)
    rapidMixThread.doneTrainingCallback = [&] ( void )
    {
        trainingText = "New Model Trained";
        printf("New Model Trained\n");
    };
    
    // Set up the bitalino
    bitalinoThread.setup("/dev/cu.BITalino-DevB", 2000,
             1000, {0, 1, 2, 3, 4, 5},
             100);
    // Start the bitalino's thread (and allow it to start searching for the device)
    bitalinoThread.startThread();
    // Start receiving data as soon as it's connected
    bitalinoThread.setRecording(true);
    // Initialize with 0, 0
    bitalinoProcessedOutput = { 0.0, 0.0 };
    // Init FFT
    fft.setup(128, 32, 32);
}

bool RapidBitalino::processBitalinoOutput(BITalino::Frame& frame)
{
    std::vector<double> output;
    double newVal = static_cast<float>(frame.analog[0]);
    
    discreteVal -= discreteVal / N;
    discreteVal += newVal / N;
    discreteValCalibrated = newVal - discreteVal;
    
    testCheapRMSdiscrete -= testCheapRMSdiscrete / M;
    testCheapRMSdiscrete += (discreteValCalibrated*discreteValCalibrated) / M;
    //bitalinoProcessedOutput = { sqrt(testCheapRMSdiscrete) };
    
    if (fft.process(discreteValCalibrated))
    {
        bitalinoProcessedOutput = { fft.spectralCentroid(), sqrt(testCheapRMSdiscrete) };
        
        return true;
    }
    return false;
}

void RapidBitalino::update ( void )
{ // TODO: move this to its own thread and create communications between ofx and this so it doesn't block ofx
    rapidMixThread.update();
    
    uint32_t items = bitalinoThread.data.items_available_for_read();
    BITalino::Frame data[items];
    bitalinoThread.data.pop(data, items);
    //double output;
    
    for (uint32_t i = 0; i < items; ++i)
    {
        if (processBitalinoOutput(data[i]))
        {
            tOutput = rapidMixThread.model->run(bitalinoProcessedOutput)[0];
            printf("Input = %f, %f, Output = %f\n",bitalinoProcessedOutput[0],bitalinoProcessedOutput[1], tOutput);
            
            if (gatheringTrainingData)
            {
                if (samplesToCollect > 0)
                {
                    (*currentPhrase).addElement(bitalinoProcessedOutput, currentTrainingOutput);
                    --samplesToCollect;
                } else {
                    gatheringTrainingData = false;
                    rapidMixThread.train(); // Start training process in its thread
                }
            }
        }
    }
}

void RapidBitalino::draw ( void )
{
    // Todo, draw some trivial debug interface items
    // ie:
    /*
     - BITalino isConnected ()
     - BITalino isRecording () // if receiving data ?
     - RapidMixThread isTraining ()
     - input and output values
     */
    ofDrawBox(ofVec2f(400,(tOutput/4)*600), 50);
}

void RapidBitalino::stop ( void )
{
    rapidMixThread.stopThread();
    bitalinoThread.stopThread();
}


